#!/bin/bash

CURRENT_VERSION="1.0"
DATE_OF_COMPILATION="23-apr-2009"
CURRENT_DIRECTORY=`pwd`
INSTALLATION_TARBALL="addit.tar"
PROJDIR_TARBALL="mirror_web.tar"
APACHE_DOCUMENT_ROOT="/var/www/html"
ADDIT_MIRRORWEB_DIR="/var/mirror_web"
SVN_REPOS_DIR="/var/www/repos"
SVN_CONFIG_DIR="/var/www/.subversion"
DB_TARBALL="db_update.tar"
CGI_TARBALL="cgi_files.tar"
CGI_BIN_DIRECTORY="/var/www/cgi-bin"
SVN_CONFIG_TARBALL="svn_files.tar"
CURRENT_DATETIME=`date +"%d-%b-%Y_%H:%m"`

if [[ $UID != "0" ]]
then
 echo "You are not root"
 echo "Only the administrator can install Add-It" 
 exit 0;
fi

echo "Add-It Installation, version $CURRENT_VERSION, $DATE_OF_COMPILATION"
echo ""

###############################################
#   STAGE 1 - CHECK PRODUCTION ENVIRONMENT    #
###############################################

echo "############################################"
echo "# Step 1/5 Checking production environment #"
echo "############################################"
echo ""

# Check if Apache is installed, and get version
echo "Verifying Apache"
APACHE_EXECUTABLE=`which httpd 2>/dev/null`
if [ $? == "1" ] 
then
echo "Apache could not be found."
exit 0;
else
APACHE_VERSION=`$APACHE_EXECUTABLE -v`
echo $APACHE_VERSION
fi
echo ""

# Check if MySQL is installed, and get version
echo "Verifying MySQL"
MYSQL_EXECUTABLE=`which mysql 2>/dev/null`
if [ $? == "1" ]
then
echo "MySQL could not be found."
exit 0;
else
MYSQL_VERSION=`$MYSQL_EXECUTABLE --version`
MYSQLDUMP_PATH=`which mysqldump`
echo $MYSQL_VERSION
fi
echo ""
# Check if Drupal is installed
echo "Verifying Drupal"
echo "You will need root access to your MySQL Server !"
echo `/usr/bin/mysql -u root -p<<SQL 
show databases; 
quit 
SQL` | tr " " "\n" | grep -s drupal
if [ $? == "1" ]
then
echo "Drupal could not be found."
exit 0;
else
echo "Drupal database was found"
fi
echo ""

# Check if SVN is installed, and get version
echo "Verifying Subversion"
SVN_EXECUTABLE=`which svn 2>/dev/null`
if [ $? == "1" ]
then
echo "Subversion could not be found."
exit 0;
else
SVN_VERSION=`$SVN_EXECUTABLE --version | head -1`
echo $SVN_VERSION
fi

echo ""
# Check if the INSTALLATION_TARBALL is present in the current directory
if [ -a "$CURRENT_DIRECTORY/$INSTALLATION_TARBALL" ]
then
echo "File $INSTALLATION_TARBALL exists"
else
echo "File $INSTALLATION_TARBALL doesn't exist !"
exit 0
fi

# Check if the PROJDIR_TARBALL is present in the current directory
if [ -a "$CURRENT_DIRECTORY/$PROJDIR_TARBALL" ]
then
echo "File $PROJDIR_TARBALL exists"
else
echo "File $PROJDIR_TARBALL doesn't exist !"
exit 0
fi

# Check if the DB_TARBALL is present in the current directory
if [ -a "$CURRENT_DIRECTORY/$DB_TARBALL" ]
then
echo "File $DB_TARBALL exists"
else
echo "File $DB_TARBALL doesn't exist !"
exit 0
fi

# Check if the CGI_TARBALL is present in the current directory
if [ -a "$CURRENT_DIRECTORY/$CGI_TARBALL" ]
then
echo "File $CGI_TARBALL exists"
else
echo "File $CGI_TARBALL doesn't exist !"
exit 0
fi


###############################################
#   STAGE 2 - CREATE DIRECTORY STRUCTURE      #
###############################################

echo "############################################"
echo "# Step 2/5 Creating directory structure    #"
echo "############################################"
echo ""

# Check if ADDIT_MIRRORWEB_DIR exists

if [ -d $ADDIT_MIRRORWEB_DIR ]
then
echo "The directory $ADDIT_MIRRORWEB_DIR already exists !"
echo "Overwrite ? [y/n]"
read YN_ANSWER
case $YN_ANSWER in
    [yY]*) echo "Overwriting $ADDIT_MIRRORWEB_DIR with new content..." 
	    rm -rf $ADDIT_MIRRORWEB_DIR
	    mkdir $ADDIT_MIRRORWEB_DIR
	    chown apache $ADDIT_MIRRORWEB_DIR
    	    ;;
    [nN]*) echo "Leaving old contents in $ADDIT_MIRRORWEB_DIR"
	    ;;
    esac

else
echo "Creating directory $ADDIT_MIRRORWEB_DIR"
mkdir $ADDIT_MIRRORWEB_DIR
chown apache $ADDIT_MIRRORWEB_DIR
fi

echo ""

# Check if the APACHE_DOCUMENT_ROOT could be sourced from Apache httpd.conf

if [ $APACHE_DOCUMENT_ROOT == "" ]
then
echo "The Apache Document root could not be identified"
echo "Please specify the Document root directory"
read APACHE_DOCUMENT_ROOT
else
echo "The Apache Document root is set to $APACHE_DOCUMENT_ROOT"
echo "Do you wish to proceed with the installation to this folder ? [y/n]"
read YN_ANSWER
case $YN_ANSWER in
    [yY]*) echo "Creating 'addit' subfolder in $APACHE_DOCUMENT_ROOT"
            mkdir $APACHE_DOCUMENT_ROOT/addit
	    chown apache $APACHE_DOCUMENT_ROOT/addit
    	    ;;
    [nN]*) echo "Please specify the Document root directory"
	   read APACHE_DOCUMENT_ROOT
           mkdir $APACHE_DOCUMENT_ROOT/addit
	   chown apache $APACHE_DOCUMENT_ROOT/addit 
	    ;;
    esac
fi

# Check where the cgi-bin directory resides...
if [ $CGI_BIN_DIRECTORY == "" ]
then
echo "The /cgi-bin/ directory doesn't seem to be configured in Apache"
echo "Please specify the path to /cgi-bin/"
read CGI_BIN_DIRECTORY
else
  if [ -d $CGI_BIN_DIRECTORY ]
  then
  echo "The /cgi-bin/ directory is ok"
  else
  echo "The /cgi-bin/ directory is not ok"
  exit 0;
  fi
echo
fi 

# Check if the /var/www/repos directory exists
if [ -d $SVN_REPOS_DIR ]
then
echo "Directory $SVN_REPOS_DIR already exists"
echo "It will be used for SVN repositaries"
else
echo "Creating directory $SVN_REPOS_DIR for SVN repositaries"
mkdir $SVN_REPOS_DIR
chown apache $SVN_REPOS_DIR
fi

# Check if the configuration directory for apache already exists
if [ -d $SVN_CONFIG_DIR ]
then
echo "Directory $SVN_CONFIG_DIR already exists"
echo "It will be used for SVN configuration"
else
echo "Creating directory $SVN_CONFIG_DIR for configuration files"
mkdir $SVN_CONFIG_DIR
chown apache $SVN_CONFIG_DIR
fi

###############################################
#   STAGE 3 - COPY DATA                       #
###############################################

echo "############################################"
echo "# Step 3/5 Copying installation data       #"
echo "############################################"
echo ""

echo "Extracting Installation data to $APACHE_DOCUMENT_ROOT/addit/"
tar -xf $CURRENT_DIRECTORY/$INSTALLATION_TARBALL -C $APACHE_DOCUMENT_ROOT/addit/

echo "Extracting ProjDir data to $ADDIT_MIRRORWEB_DIR/"
tar -xf $CURRENT_DIRECTORY/$PROJDIR_TARBALL -C $ADDIT_MIRRORWEB_DIR/

echo "Extracting CGI-BIN data to $CGI_BIN_DIRECTORY/"
tar -xf $CURRENT_DIRECTORY/$CGI_TARBALL -C $CGI_BIN_DIRECTORY/

echo "Extracting DB-Update data to $CURRENT_DIRECTORY/"
tar -xf $CURRENT_DIRECTORY/$DB_TARBALL -C $CURRENT_DIRECTORY/

echo "Extracting SVN configuration files to $SVN_CONFIG_DIR/"
tar -xf $CURRENT_DIRECTORY/$SVN_CONFIG_TARBALL -C $SVN_CONFIG_DIR

###############################################
#   STAGE 4 - UPDATE DATABASES                #
###############################################

echo "############################################"
echo "# Step 4/5 Updating MySQL Database         #"
echo "############################################"
echo ""

echo "Do you wish to back the MySQL database content at this time ? [y/n]"
read YN_ANSWER
case $YN_ANSWER in
    [yY]*) echo "Backing up DB, please provide root password for MySQL"
	   $MYSQLDUMP_PATH -u root -p --all-databases > db_backup.$CURRENT_DATETIME.dump
	   echo "The entire database was backed up to $CURRENT_DIRECTORY/db_backup.$CURRENT_DATETIME.dump"
            ;;
    [nN]*) echo "Database has not been dumped to a backup..."
            ;;
    esac

#echo "Updating Drupal DB"
#echo "You need to provide the root password for MySQL"
#mysql -u root -p cms_drupal < $CURRENT_DIRECTORY/drupal_update.sql 

echo ""

echo "Creating Add-It Database structure"
echo "You need to provide the root password for MySQL"
mysql -u root -p <<SQL
CREATE DATABASE mirror_web;
quit
SQL
echo "Updating MySQL tables in mirror_web db"
mysql -u root -p mirror_web < $CURRENT_DIRECTORY/addit_update.sql

echo "Creating addit user in MySQL"
/usr/bin/mysql -u root -p<<SQL
CREATE USER 'addit'@'localhost' IDENTIFIED BY 'fw@g50=1.';
GRANT SELECT,INSERT,UPDATE,DELETE,CREATE,DROP on mirror_web.* TO 'addit'@'localhost';
quit
SQL

echo "Installation has finished !"
echo ""
echo "Please add this cronjob for user Apache :"
echo "0,15,30,45 * * * * $CGI_BIN_DIRECTORY/clear_temp.sh > /dev/null"
